@@ -28,6 +28,8 @@ COpData::COpData(enum Command op_Id)
 	pNextOpData = 0;
 
 	waitForAsyncRequest = false;
+
+	holdsLock = false;
 }
 
 COpData::~COpData()
@@ -48,7 +50,7 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate *pEngine)
 	m_useUTF8 = false;
 
 	m_timer.SetOwner(this);
-	
+
 	m_closed = false;
 
 	m_invalidateCurrentPath = false;
@@ -87,10 +89,8 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		LogMessage(::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
 	}
 
-	if (m_pCurOpData && m_pCurOpData->opId != cmd_rawtransfer)
-	{
+	if (m_pCurOpData && m_pCurOpData->holdsLock)
 		UnlockCache();
-	}
 
 	if (m_pCurOpData && m_pCurOpData->pNextOpData)
 	{
@@ -184,11 +184,11 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 		wxASSERT(!m_pCurOpData);
 		return nErrorCode;
 	}
-	
+
 	m_closed = true;
 
 	nErrorCode = ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED | nErrorCode);
-	
+
 	delete m_pCurrentServer;
 	m_pCurrentServer = 0;
 
@@ -255,6 +255,7 @@ void CControlSocket::InitTransferStatus(wxFileOffset totalSize, wxFileOffset sta
 	m_pTransferStatus->totalSize = totalSize;
 	m_pTransferStatus->startOffset = startOffset;
 	m_pTransferStatus->currentOffset = startOffset;
+	m_pTransferStatus->madeProgress = false;
 }
 
 void CControlSocket::SetTransferStatusStartTime()
@@ -265,6 +266,14 @@ void CControlSocket::SetTransferStatusStartTime()
 	m_pTransferStatus->started = wxDateTime::Now();
 }
 
+void CControlSocket::SetTransferStatusMadeProgress()
+{
+	if (!m_pTransferStatus)
+		return;
+
+	m_pTransferStatus->madeProgress = true;
+}
+
 void CControlSocket::UpdateTransferStatus(wxFileOffset transferredBytes)
 {
 	if (!m_pTransferStatus)
@@ -315,22 +324,22 @@ bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, con
 		int pos2 = reply.Find('"', true);
 		if (pos1 == -1 || pos1 >= pos2)
 		{
-			int pos1 = reply.Find('\'');
-			int pos2 = reply.Find('\'', true);
+			pos1 = reply.Find('\'');
+			pos2 = reply.Find('\'', true);
 
 			if (pos1 != -1 && pos1 < pos2)
 				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server sending single-quoted path instead of double-quoted path."));
 		}
 		if (pos1 == -1 || pos1 >= pos2)
 		{
-			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No quoted path found in pwd reply, trying first token as path"));
+			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
 			pos1 = reply.Find(' ');
 			if (pos1 != -1)
 			{
-				pos2 = reply.Mid(pos1 + 1).Find(' ');
-				if (pos2 == -1)
-					pos2 = (int)reply.Length();
-				reply = reply.Mid(pos1 + 1, pos2 - pos1 - 1);
+				reply = reply.Mid(pos1 + 1);
+				pos2 = reply.Find(' ');
+				if (pos2 != -1)
+					reply = reply.Left(pos2);
 			}
 			else
 				reply = _T("");
@@ -407,6 +416,13 @@ int CControlSocket::CheckOverwriteFile()
 	pNotification->remoteSize = pData->remoteFileSize;
 	pNotification->ascii = !pData->transferSettings.binary;
 
+	if (pData->download && pNotification->localSize != -1)
+		pNotification->canResume = true;
+	else if (!pData->download && pNotification->remoteSize != -1)
+		pNotification->canResume = true;
+	else
+		pNotification->canResume = false;
+
 	wxStructStat buf;
 	int result;
 	result = wxStat(pData->localFile, &buf);
@@ -432,10 +448,7 @@ int CControlSocket::CheckOverwriteFile()
 		}
 	}
 
-	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
-	pData->waitForAsyncRequest = true;
-
-	m_pEngine->AddNotification(pNotification);
+	SendAsyncRequest(pNotification);
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -470,7 +483,7 @@ wxString CControlSocket::ConvToLocal(const char* buffer)
 			m_useUTF8 = false;
 		}
 	}
-	
+
 	if (m_pCSConv)
 	{
 		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv);
@@ -647,9 +660,10 @@ const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockSta
 	return iter;
 }
 
-bool CControlSocket::TryLockCache(const CServerPath& directory)
+bool CControlSocket::TryLockCache(enum locking_reason reason, const CServerPath& directory)
 {
 	wxASSERT(m_pCurrentServer);
+	wxASSERT(m_pCurOpData);
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
@@ -658,38 +672,91 @@ bool CControlSocket::TryLockCache(const CServerPath& directory)
 		info.directory = directory;
 		info.pControlSocket = this;
 		info.waiting = true;
+		info.reason = reason;
+		info.lockcount = 0;
 		m_lockInfoList.push_back(info);
 		own = --m_lockInfoList.end();
 	}
 	else
 	{
+		if (own->lockcount)
+		{
+			if (!m_pCurOpData->holdsLock)
+			{
+				m_pCurOpData->holdsLock = true;
+				own->lockcount++;
+			}
+			return true;
+		}
 		wxASSERT(own->waiting);
+		wxASSERT(own->reason == reason);
 	}
 
+	// Needs to be set in any case so that ResetOperation 
+	// unlocks or cancels the lock wait
+	m_pCurOpData->holdsLock = true;
+
 	// Try to find other instance holding the lock
 	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; iter++)
 	{
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
+		if (directory != iter->directory)
+			continue;
+		if (reason != iter->reason)
+			continue;
 
-		if (directory == iter->directory)
-		{
-			// Some other instance is holding the lock
-			return false;
-		}
+		// Some other instance is holding the lock
+		return false;
 	}
 
+	own->lockcount++;
 	own->waiting = false;
 	return true;
 }
 
+bool CControlSocket::IsLocked(enum locking_reason reason, const CServerPath& directory)
+{
+	wxASSERT(m_pCurrentServer);
+
+	std::list<t_lockInfo>::iterator own = GetLockStatus();
+	if (own != m_lockInfoList.end())
+		return true;
+
+	// Try to find other instance holding the lock
+	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; iter++)
+	{
+		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
+			continue;
+		if (directory != iter->directory)
+			continue;
+		if (reason != iter->reason)
+			continue;
+
+		// Some instance is holding the lock
+		return true;
+	}
+
+	return false;
+}
+
 void CControlSocket::UnlockCache()
 {
+	if (!m_pCurOpData || !m_pCurOpData->holdsLock)
+		return;
+	m_pCurOpData->holdsLock = false;
+
 	std::list<t_lockInfo>::iterator iter = GetLockStatus();
 	if (iter == m_lockInfoList.end())
 		return;
 
+	iter->lockcount--;
+	if (iter->lockcount)
+		return;
+
 	CServerPath directory = iter->directory;
+	enum locking_reason reason = iter->reason;
+
 	m_lockInfoList.erase(iter);
 
 	// Find other instance waiting for the lock
@@ -701,6 +768,9 @@ void CControlSocket::UnlockCache()
 		if (iter->directory != directory)
 			continue;
 
+		if (iter->reason != reason)
+			continue;
+
 		// Send notification
 		wxCommandEvent evt(fzOBTAINLOCK);
 		iter->pControlSocket->AddPendingEvent(evt);
@@ -708,44 +778,41 @@ void CControlSocket::UnlockCache()
 	}
 }
 
-bool CControlSocket::ObtainLockFromEvent()
+enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 {
+	wxASSERT(m_pCurOpData);
+
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
-		return false;
+		return lock_unknown;
 
 	if (!own->waiting)
-		return false;
+		return lock_unknown;
 
 	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != own; iter++)
 	{
 		if (*m_pCurrentServer != *iter->pControlSocket->m_pCurrentServer)
 			continue;
 
-		if (iter->directory == own->directory)
-			return false;
+		if (iter->directory != own->directory)
+			continue;
+
+		if (iter->reason != own->reason)
+			continue;
+
+		// Another instance comes before us
+		return lock_unknown;
 	}
 
 	own->waiting = false;
+	own->lockcount++;
 
-	return true;
-}
-
-bool CControlSocket::HasLock()
-{
-	std::list<t_lockInfo>::iterator own = GetLockStatus();
-	if (own == m_lockInfoList.end())
-		return false;
-
-	if (own->waiting)
-		return false;
-
-	return true;
+	return own->reason;
 }
 
 void CControlSocket::OnObtainLock(wxCommandEvent& event)
 {
-	if (!ObtainLockFromEvent())
+	if (ObtainLockFromEvent() == lock_unknown)
 		return;
 
 	SendNextCommand();
@@ -776,10 +843,21 @@ wxTimeSpan CControlSocket::GetTimezoneOffset()
 	int seconds = 0;
 	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) != yes)
 		return wxTimeSpan();
-	
+
 	return wxTimeSpan(0, 0, seconds);
 }
 
+void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
+{
+	wxASSERT(pNotification);
+
+	pNotification->requestNumber = m_pEngine->GetNextAsyncRequestNumber();
+
+	if (m_pCurOpData)
+		m_pCurOpData->waitForAsyncRequest = true;
+	m_pEngine->AddNotification(pNotification);
+}
+
 // ------------------
 // CRealControlSocket
 // ------------------
@@ -953,7 +1031,7 @@ int CRealControlSocket::Connect(const CServer &server)
 		pData = 0;
 	else
 		pData = static_cast<CConnectOpData *>(m_pCurOpData);
-	
+
 	const wxString& host = pData ? pData->host : server.GetHost();
 	if (!IsIpAddress(host))
 	{
@@ -983,7 +1061,7 @@ int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
 		LogMessage(Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
 		return DoClose(FZ_REPLY_INTERNALERROR);
 	}
-	
+
 	if (!address)
 	{
 		LogMessage(::Error, _("Invalid hostname or host not found"));
@@ -1013,7 +1091,7 @@ int CRealControlSocket::ContinueConnect(const wxIPV4address *address)
 int CRealControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
 	ResetSocket();
-	
+
 	return CControlSocket::DoClose(nErrorCode);
 }
 
