@@ -26,10 +26,13 @@ protected:
 
 		bool operator<(const CSourcePath& op) const
 		{
-			if (source.GetSafePath() < op.source.GetSafePath())
+			const int cmp = subdir.Cmp(op.subdir);
+			if (cmp < 0)
 				return true;
+			if (cmp > 0)
+				return false;
 
-			return subdir < op.subdir;
+			return source < op.source;
 		}
 	};
 	
