@@ -6,6 +6,7 @@
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "inputdialog.h"
+#include "local_filesys.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -202,14 +203,18 @@ EVT_MENU(XRCID("ID_RENAME"), CLocalTreeView::OnMenuRename)
 EVT_MENU(XRCID("ID_MKDIR"), CLocalTreeView::OnMenuMkdir)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnEndLabelEdit)
+EVT_CHAR(CLocalTreeView::OnChar)
 END_EVENT_TABLE()
 
 CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState, CQueueView *pQueueView)
 	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER),
 	CSystemImageList(16),
-	CStateEventHandler(pState, STATECHANGE_LOCAL_DIR | STATECHANGE_APPLYFILTER),
+	CStateEventHandler(pState),
 	m_pQueueView(pQueueView)
 {
+	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+
 	m_setSelection = false;
 
 	SetImageList(GetSystemImageList());
@@ -439,11 +444,11 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir /*=_T("")*/)
 {
-	wxDir dir;
+	CLocalFileSystem local_filesystem;
 
 	{
 		wxLogNull log;
-		if (!dir.Open(dirname))
+		if (!local_filesystem.BeginFindFiles(dirname, true))
 		{
 			if (knownSubdir != _T(""))
 			{
@@ -471,14 +476,18 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 	DeleteChildren(parent);
 	m_setSelection = false;
 
-	wxString file;
-
-	CFilterDialog filter;
+	CFilterManager filter;
 
 	bool matchedKnown = false;
 
-	for (bool found = dir.GetFirst(&file, _T(""), wxDIR_DIRS | wxDIR_HIDDEN); found; found = dir.GetNext(&file))
+	wxString file;
+	bool wasLink;
+	int attributes;
+	bool is_dir;
+	const wxLongLong size(-1);
+	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, 0, &attributes))
 	{
+		wxASSERT(is_dir);
 		if (file == _T(""))
 		{
 			wxGetApp().DisplayEncodingWarning();
@@ -492,34 +501,7 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		if (file != knownSubdir)
 #endif
 		{
-			wxFileName fn(fullName);
-			const bool isDir = fn.DirExists();
-
-			wxLongLong size;
-
-			const wxString& fullName = fn.GetFullPath();
-			wxStructStat buf;
-			int result = wxStat(fullName, &buf);
-			if (!isDir && !result)
-				size = buf.st_size;
-			else
-				size = -1;
-
-			int attributes;
-#ifdef __WXMSW__
-			DWORD tmp = GetFileAttributes(fullName);
-			if (tmp == INVALID_FILE_ATTRIBUTES)
-				attributes = 0;
-			else
-				attributes = tmp;
-#else
-			if (!result)
-				attributes = buf.st_mode & 0x777;
-			else
-				attributes = -1;
-#endif //__WXMSW__
-
-			if (filter.FilenameFiltered(file, isDir, size, true, attributes))
+			if (filter.FilenameFiltered(file, true, size, true, attributes))
 				continue;
 		}
 		else
@@ -544,31 +526,32 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 bool CLocalTreeView::HasSubdir(const wxString& dirname)
 {
 	wxLogNull nullLog;
-	wxDir dir(dirname);
-	if (!dir.IsOpened())
+
+	CFilterManager filter;
+	
+	CLocalFileSystem local_filesystem;
+	if (!local_filesystem.BeginFindFiles(dirname, true))
 		return false;
 
-	CFilterDialog filter;
 	wxString file;
-	for (bool found = dir.GetFirst(&file, _T(""), wxDIR_DIRS | wxDIR_HIDDEN); found; found = dir.GetNext(&file))
+	bool wasLink;
+	int attributes;
+	bool is_dir;
+	const wxLongLong size(-1);
+	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, 0, &attributes))
 	{
-		int attributes;
-#ifdef __WXMSW__
-		DWORD tmp = GetFileAttributes(dirname + wxFileName::GetPathSeparator() + file);
-		if (tmp == INVALID_FILE_ATTRIBUTES)
-			attributes = 0;
-		else
-			attributes = tmp;
-#else
-		wxStructStat buf;
-		const int result = wxLstat(dirname + file, &buf);
-		if (!result)
-			attributes = buf.st_mode & 0x777;
-		else
-			attributes = -1;
-#endif //__WXMSWMM
-		if (!filter.FilenameFiltered(file, true, -1, true, attributes))
-			return true;
+		wxASSERT(is_dir);
+		if (file == _T(""))
+		{
+			wxGetApp().DisplayEncodingWarning();
+			continue;
+		}
+
+		wxString fullName = dirname + file;
+		if (filter.FilenameFiltered(file, true, size, true, attributes))
+			continue;
+
+		return true;
 	}
 
 	return false;
@@ -729,15 +712,15 @@ void CLocalTreeView::Refresh()
 	dirsToCheck.push_back(dir);
 #endif
 
-	CFilterDialog filter;
+	CFilterManager filter;
 
 	while (!dirsToCheck.empty())
 	{
 		t_dir dir = dirsToCheck.front();
 		dirsToCheck.pop_front();
 
-		wxDir find(dir.dir);
-		if (!find.IsOpened())
+		CLocalFileSystem local_filesystem;
+		if (!local_filesystem.BeginFindFiles(dir.dir, true))
 		{
 			// Dir does exist (listed in parent) but may not be accessible.
 			// Recurse into children anyhow, they might be accessible again.
@@ -755,48 +738,28 @@ void CLocalTreeView::Refresh()
 			continue;
 		}
 
-		wxString file;
-		bool found = find.GetFirst(&file, _T(""), wxDIR_DIRS | wxDIR_HIDDEN);
 		std::list<wxString> dirs;
-		while (found)
+
+		
+		wxString file;
+		const wxLongLong size(-1);
+		bool was_link;
+		bool is_dir;
+		int attributes;
+		while (local_filesystem.GetNextFile(file, was_link, is_dir, 0, 0, &attributes))
 		{
 			if (file == _T(""))
 			{
 				wxGetApp().DisplayEncodingWarning();
-				found = find.GetNext(&file);
 				continue;
 			}
-			wxFileName fn(dir.dir + file);
-			const bool isDir = fn.DirExists();
 
-			wxLongLong size;
+			if (filter.FilenameFiltered(file, true, size, true, attributes))
+				continue;
 
-			wxStructStat buf;
-			int result = wxStat(fn.GetFullPath(), &buf);
-			if (!isDir && !result)
-				size = buf.st_size;
-			else
-				size = -1;
-
-			int attributes;
-#ifdef __WXMSW__
-			DWORD tmp = GetFileAttributes(fn.GetFullPath());
-			if (tmp == INVALID_FILE_ATTRIBUTES)
-				attributes = 0;
-			else
-				attributes = tmp;
-#else
-			if (!result)
-				attributes = buf.st_mode & 0x777;
-			else
-				attributes = -1;
-#endif //__WXMSW__
-
-			if (!filter.FilenameFiltered(file, isDir, size , true, attributes))
-				dirs.push_back(file);
-			found = find.GetNext(&file);
+			dirs.push_back(file);
 		}
-		dirs.sort(sortfunc);
+		dirs.sort(sortfunc);		
 
 		bool inserted = false;
 
@@ -928,12 +891,15 @@ int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId
 #endif
 }
 
-void CLocalTreeView::OnStateChange(unsigned int event, const wxString& data)
+void CLocalTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
-	if (event == STATECHANGE_LOCAL_DIR)
+	if (notification == STATECHANGE_LOCAL_DIR)
 		SetDir(m_pState->GetLocalDir());
-	else if (event == STATECHANGE_APPLYFILTER)
+	else
+	{
+		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		Refresh();
+	}
 }
 
 void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
@@ -951,8 +917,12 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	if (dir == _T("/"))
 		return;
 
+#ifdef __WXMSW__
 	if (dir.Last() == '\\')
 		dir.RemoveLast();
+#endif
+	if (dir.Last() == '/')
+		dir.RemoveLast();
 
 #ifdef __WXMSW__
 	if (dir.Last() == ':')
@@ -1400,84 +1370,8 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 		return;
 #endif
 
-	// Under Windows use SHFileOperation to delete files and directories.
-	// Under other systems, we have to recurse into subdirectories manually
-	// to delete all contents.
-
-#ifdef __WXMSW__
-	// SHFileOperation accepts a list of null-terminated strings. Go through all
-	// items to get the required buffer length
-
-	if (path.Last() == wxFileName::GetPathSeparator())
-		path.RemoveLast();
-
-	int len = path.Len() + 2;
-
-	// Allocate memory
-	wxChar* pBuffer = new wxChar[len];
-	_tcscpy(pBuffer, path);
-	pBuffer[len - 1] = 0;
-
-	// Now we can delete the files in the buffer
-	SHFILEOPSTRUCT op;
-	memset(&op, 0, sizeof(op));
-	op.hwnd = (HWND)GetHandle();
-	op.wFunc = FO_DELETE;
-	op.pFrom = pBuffer;
-
-	// Move to trash if shift is not pressed, else delete
-	op.fFlags = wxGetKeyState(WXK_SHIFT) ? 0 : FOF_ALLOWUNDO;
-
-	SHFileOperation(&op);
-
-	delete [] pBuffer;
-#else
-	if (wxMessageBox(_("Really delete all selected files and/or directories?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
-		return;
-
-	// Remember the directories to delete and the directories to visit
-	std::list<wxString> dirsToDelete;
-	std::list<wxString> dirsToVisit;
-
-	if (path.Last() != wxFileName::GetPathSeparator())
-		path += wxFileName::GetPathSeparator();
-
-	dirsToVisit.push_back(path);
-	dirsToDelete.push_front(path);
-
-	// Process any subdirs which still have to be visited
-	while (!dirsToVisit.empty())
-	{
-		wxString filename = dirsToVisit.front();
-		dirsToVisit.pop_front();
-		wxDir dir;
-		if (!dir.Open(filename))
-			continue;
-
-		wxString file;
-		for (bool found = dir.GetFirst(&file); found; found = dir.GetNext(&file))
-		{
-			if (file == _T(""))
-			{
-				wxGetApp().DisplayEncodingWarning();
-				continue;
-			}
-			if (wxFileName::DirExists(filename + file))
-			{
-				wxString subDir = filename + file + wxFileName::GetPathSeparator();
-				dirsToVisit.push_back(subDir);
-				dirsToDelete.push_front(subDir);
-			}
-			else
-				wxRemoveFile(filename + file);
-		}
-	}
-
-	// Delete the now empty directories
-	for (std::list<wxString>::const_iterator iter = dirsToDelete.begin(); iter != dirsToDelete.end(); iter++)
-		wxRmdir(*iter);
-
-#endif
+	if (!CLocalFileSystem::RecursiveDelete(path, this))
+		wxGetApp().DisplayEncodingWarning();
 
 	wxTreeItemId item = GetSelection();
 	while (item && item != m_contextMenuItem)
@@ -1584,6 +1478,12 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(item);
 	const wxString& newName = event.GetLabel();
+	if (newName == _T(""))
+	{
+		wxBell();
+		event.Veto();
+		return;
+	}
 
 	wxASSERT(parent + oldName == path);
 
@@ -1627,3 +1527,16 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	// Current selection below renamed item
 	m_pState->SetLocalDir(parent + newName + sub);
 }
+
+void CLocalTreeView::OnChar(wxKeyEvent& event)
+{
+	m_contextMenuItem = GetSelection();
+
+	wxCommandEvent cmdEvt;
+	if (event.GetKeyCode() == WXK_F2)
+		OnMenuRename(cmdEvt);
+	else if (event.GetKeyCode() == WXK_DELETE)
+		OnMenuDelete(cmdEvt);
+	else
+		event.Skip();
+}
