@@ -44,8 +44,12 @@ public:
 		const int dropTarget = m_pRemoteListView->m_dropTarget;
 		if (dropTarget != -1)
 		{
-			m_pRemoteListView->SetItemState(dropTarget, 0, wxLIST_STATE_DROPHILITED);
 			m_pRemoteListView->m_dropTarget = -1;
+#ifdef __WXMSW__
+			m_pRemoteListView->SetItemState(dropTarget, 0, wxLIST_STATE_DROPHILITED);
+#else
+			m_pRemoteListView->RefreshItem(dropTarget);
+#endif
 		}
 	}
 
@@ -70,7 +74,7 @@ public:
 			if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM))
 			{
 				int index = m_pRemoteListView->GetItemIndex(hit);
-				if (index != -1)
+				if (index != -1 && m_pRemoteListView->m_fileData[index].flags != CComparableListing::fill)
 				{
 					if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 						subdir = _T("..");
@@ -104,7 +108,7 @@ public:
 		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM))
 		{
 			int index = m_pRemoteListView->GetItemIndex(hit);
-			if (index != -1)
+			if (index != -1 && m_pRemoteListView->m_fileData[index].flags != CComparableListing::fill)
 			{
 				if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 					subdir = _T("..");
@@ -180,7 +184,7 @@ public:
 		if (hit != -1)
 		{
 			int index = m_pRemoteListView->GetItemIndex(hit);
-			if (index == -1)
+			if (index == -1 || m_pRemoteListView->m_fileData[index].flags == CComparableListing::fill)
 				hit = -1;
 			else if (index != (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 			{
@@ -202,8 +206,12 @@ public:
 			ClearDropHighlight();
 			if (hit != -1)
 			{
-				m_pRemoteListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
 				m_pRemoteListView->m_dropTarget = hit;
+#ifdef __WXMSW__
+				m_pRemoteListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
+#else
+				m_pRemoteListView->RefreshItem(hit);
+#endif
 			}
 		}
 		return hit;
@@ -289,6 +297,7 @@ protected:
 		wxPaintDC paintDc(this);
 
 		paintDc.SetFont(GetFont());
+		paintDc.SetTextForeground(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
 
 		paintDc.DrawText(m_text, 0, 0);
 	};
@@ -324,38 +333,28 @@ END_EVENT_TABLE()
 CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
 	: CFileListCtrl<CGenericFileData>(pParent, pState, pQueue),
 	CSystemImageList(16),
-	CStateEventHandler(pState, STATECHANGE_REMOTE_DIR | STATECHANGE_REMOTE_DIR_MODIFIED | STATECHANGE_APPLYFILTER)
+	CStateEventHandler(pState)
 {
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
+	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+
 	m_dropTarget = -1;
 
 	m_pInfoText = 0;
 	m_pDirectoryListing = 0;
 
-	unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
+	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
 
-	if (!wxGetKeyState(WXK_SHIFT) || !wxGetKeyState(WXK_ALT) || !wxGetKeyState(WXK_CONTROL))
-		COptions::Get()->ReadColumnWidths(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, 6, widths);
+	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
+	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
+	AddColumn(_("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
+	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
+	AddColumn(_("Permissions"), wxLIST_FORMAT_LEFT, widths[4]);
+	AddColumn(_("Owner / Group"), wxLIST_FORMAT_LEFT, widths[5]);
+	LoadColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
-	InsertColumn(0, _("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
-	InsertColumn(1, _("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
-	InsertColumn(2, _("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
-	InsertColumn(3, _("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
-	InsertColumn(4, _("Permissions"), wxLIST_FORMAT_LEFT, widths[4]);
-	InsertColumn(5, _("Owner / Group"), wxLIST_FORMAT_LEFT, widths[5]);
-
-	wxString sortInfo = COptions::Get()->GetOption(OPTION_REMOTEFILELIST_SORTORDER);
-	m_sortDirection = sortInfo[0] - '0';
-	if (m_sortDirection < 0 || m_sortDirection > 1)
-		m_sortDirection = 0;
-
-	if (sortInfo.Len() == 3)
-	{
-		m_sortColumn = sortInfo[2] - '0';
-		if (m_sortColumn < 0 || m_sortColumn > 5)
-			m_sortColumn = 0;
-	}
-	else
-		m_sortColumn = 0;
+	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
 
 	m_dirIcon = GetIconIndex(dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
@@ -433,7 +432,8 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 	unsigned int i = 0;
 	while (i < pDirectoryListing->GetCount() && j < m_pDirectoryListing->GetCount())
 	{
-		const wxString& oldName = (*m_pDirectoryListing)[j].name;
+		const CDirentry& oldEntry = (*m_pDirectoryListing)[j];
+		const wxString& oldName = oldEntry.name;
 		const wxString& newName = (*pDirectoryListing)[i].name;
 		if (oldName == newName)
 		{
@@ -442,6 +442,14 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 			continue;
 		}
 
+		if (m_pFilelistStatusBar)
+		{
+			if (oldEntry.dir)
+				m_pFilelistStatusBar->RemoveDirectory();
+			else
+				m_pFilelistStatusBar->RemoveFile(oldEntry.size);
+		}
+
 		removedItems.push_back(j++);
 	}
 	for (; j < m_pDirectoryListing->GetCount(); j++)
@@ -449,12 +457,6 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 
 	wxASSERT(removedItems.size() == removed);
 
-	// Remove corresponding file data
-	for (std::list<unsigned int>::reverse_iterator iter = removedItems.rbegin(); iter != removedItems.rend(); iter++)
-	{
-		m_fileData.erase(m_fileData.begin() + *iter);
-	}
-
 	std::list<int> selectedItems;
 
 	// Number of items left to remove
@@ -467,14 +469,16 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 	{
 		bool removed = false;
 
+		unsigned int& index = m_indexMapping[i];
+
 		// j is the offset the index has to be adjusted
 		int j = 0;
 		for (std::list<unsigned int>::const_iterator iter = removedItems.begin(); iter != removedItems.end(); iter++, j++)
 		{
-			if (*iter > m_indexMapping[i])
+			if (*iter > index)
 				break;
 
-			if (*iter == m_indexMapping[i])
+			if (*iter == index)
 			{
 				removedIndexes.push_back(i);
 				removed = true;
@@ -482,10 +486,23 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 				break;
 			}
 		}
-		m_indexMapping[i] -= j;
 
-		// Update selection
+		// Get old selection
 		bool isSelected = GetItemState(i, wxLIST_STATE_SELECTED) != 0;
+		
+		// Update statusbar info
+		if (isSelected && removed && m_pFilelistStatusBar)
+		{
+			if ((*m_pDirectoryListing)[index].dir)
+				m_pFilelistStatusBar->UnselectDirectory();
+			else
+				m_pFilelistStatusBar->UnselectFile((*m_pDirectoryListing)[index].size);
+		}
+
+		// Update index
+		index -= j;
+
+		// Update selections
 		bool needSelection;
 		if (selectedItems.empty())
 			needSelection = false;
@@ -499,16 +516,23 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 
 		if (isSelected)
 		{
-			if (!needSelection && toRemove)
-				SetItemState(i, 0, wxLIST_STATE_SELECTED);
+			if (!needSelection && (toRemove || removed))
+				SetSelection(i, false);
 
 			if (!removed)
 				selectedItems.push_back(i - toRemove);
 		}
 		else if (needSelection)
-			SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+			SetSelection(i, true);
 	}
 
+	// Erase file data
+	for (std::list<unsigned int>::reverse_iterator iter = removedItems.rbegin(); iter != removedItems.rend(); iter++)
+	{
+		m_fileData.erase(m_fileData.begin() + *iter);
+	}
+
+	// Erase indexes
 	wxASSERT(!toRemove);
 	wxASSERT(removedIndexes.size() == removed);
 	for (std::list<int>::iterator iter = removedIndexes.begin(); iter != removedIndexes.end(); iter++)
@@ -520,13 +544,25 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CDirectoryListing *pD
 
 	m_pDirectoryListing = pDirectoryListing;
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->GetCount() + 1 - m_indexMapping.size());
+
 	SetItemCount(m_indexMapping.size());
 }
 
 bool CRemoteListView::UpdateDirectoryListing(const CDirectoryListing *pDirectoryListing)
 {
 	wxASSERT(!IsComparing());
-	if ((pDirectoryListing->m_hasUnsureEntries & UNSURE_CHANGE) == UNSURE_CHANGE)
+
+	const int unsure = pDirectoryListing->m_hasUnsureEntries & ~(CDirectoryListing::unsure_unknown);
+
+	if (!unsure)
+		return false;
+
+	if (unsure & CDirectoryListing::unsure_invalid)
+		return false;
+
+	if (!(unsure & ~(CDirectoryListing::unsure_dir_changed | CDirectoryListing::unsure_file_changed)))
 	{
 		if (m_sortColumn && m_sortColumn != 2)
 		{
@@ -534,23 +570,25 @@ bool CRemoteListView::UpdateDirectoryListing(const CDirectoryListing *pDirectory
 			// sort order.
 			return false;
 		}
-	}
+		wxASSERT(pDirectoryListing->GetCount() == m_pDirectoryListing->GetCount());
+		if (pDirectoryListing->GetCount() != m_pDirectoryListing->GetCount())
+			return false;
+		
+		m_pDirectoryListing = pDirectoryListing;
 
-	if ((pDirectoryListing->m_hasUnsureEntries & UNSURE_CONFUSED) == UNSURE_CONFUSED)
-		return false;
-
-	int addremove = UNSURE_ADD | UNSURE_REMOVE;
-	if ((pDirectoryListing->m_hasUnsureEntries & addremove) == addremove)
-		return false;
-
-	if ((pDirectoryListing->m_hasUnsureEntries & UNSURE_REMOVE) == UNSURE_REMOVE)
-	{
-		wxASSERT(pDirectoryListing->GetCount() < m_pDirectoryListing->GetCount());
-		UpdateDirectoryListing_Removed(pDirectoryListing);
+		// We don't have to do anything
 		return true;
 	}
 
-	return false;
+	if (unsure & (CDirectoryListing::unsure_dir_added | CDirectoryListing::unsure_file_added))
+	{
+		// Todo: Would be nice to handle this too
+		return false;
+	}
+
+	wxASSERT(pDirectoryListing->GetCount() < m_pDirectoryListing->GetCount());
+	UpdateDirectoryListing_Removed(pDirectoryListing);
+	return true;
 }
 
 void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryListing, bool modified /*=false*/)
@@ -564,8 +602,11 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 		reset = true;
 	else if (m_pDirectoryListing->path != pDirectoryListing->path)
 		reset = true;
-	else if (m_pDirectoryListing->GetCount() > 200 && m_pDirectoryListing->m_firstListTime == pDirectoryListing->m_firstListTime &&
-		!IsComparing())
+	else if (m_pDirectoryListing->m_firstListTime == pDirectoryListing->m_firstListTime && !IsComparing()
+#ifndef __WXDEBUG__
+		&& m_pDirectoryListing->GetCount() > 200
+#endif
+		)
 	{
 		// Updated directory listing. Check if we can use process it in a different,
 		// more efficient way.
@@ -587,6 +628,8 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 	std::list<wxString> selectedNames;
 	if (reset)
 	{
+		ResetSearchPrefix();
+
 		if (IsComparing())
 			ExitComparisonMode();
 
@@ -597,7 +640,7 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 			if (item == -1)
 				break;
-			SetItemState(item, 0, wxLIST_STATE_SELECTED);
+			SetSelection(item, false);
 		}
 		item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 		if (item != -1)
@@ -612,11 +655,20 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 		selectedNames = RememberSelectedItems(prevFocused);
 	}
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->UnselectAll();
+
 	m_pDirectoryListing = pDirectoryListing;
 
 	m_fileData.clear();
 	m_indexMapping.clear();
 
+	wxLongLong totalSize;
+	int unknown_sizes = 0;
+	int totalFileCount = 0;
+	int totalDirCount = 0;
+	int hidden = 0;
+
 	bool eraseBackground = false;
 	if (m_pDirectoryListing)
 	{
@@ -624,7 +676,7 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 
 		m_indexMapping.push_back(m_pDirectoryListing->GetCount());
 
-		CFilterDialog filter;
+		CFilterManager filter;
 		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); i++)
 		{
 			const CDirentry& entry = (*m_pDirectoryListing)[i];
@@ -633,8 +685,24 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 			data.icon = entry.dir ? m_dirIcon : -2;
 			m_fileData.push_back(data);
 
-			if (!filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
-				m_indexMapping.push_back(i);
+			if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
+			{
+				hidden++;
+				continue;
+			}
+
+			if (entry.dir)
+				totalDirCount++;
+			else
+			{
+				if (entry.size == -1)
+					unknown_sizes++;
+				else
+					totalSize += entry.size;
+				totalFileCount++;
+			}
+
+			m_indexMapping.push_back(i);
 		}
 
 		CGenericFileData data;
@@ -648,6 +716,9 @@ void CRemoteListView::SetDirectoryListing(const CDirectoryListing *pDirectoryLis
 		SetInfoText();
 	}
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+
 	if (m_dropTarget != -1)
 	{
 		bool resetDropTarget = false;
@@ -1038,6 +1109,12 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 {
+	if (GetEditControl())
+	{
+		event.Skip();
+		return;
+	}
+
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTEFILELIST"));
 	if (!pMenu)
 		return;
@@ -1242,9 +1319,18 @@ void CRemoteListView::OnMenuMkdir(wxCommandEvent& event)
 		dlg.SelectText(pos, pos + newName.Length());
 	}
 
+	const CServerPath oldPath = m_pDirectoryListing->path;
+
 	if (dlg.ShowModal() != wxID_OK)
 		return;
 
+	if (!m_pDirectoryListing || oldPath != m_pDirectoryListing->path ||
+		!m_pState->IsRemoteIdle())
+	{
+		wxBell();
+		return;
+	}
+
 	path = m_pDirectoryListing->path;
 	if (!path.ChangePath(dlg.GetValue()))
 	{
@@ -1410,10 +1496,12 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 		{
 			int index = GetItemIndex(i);
 			if (index != -1 && m_fileData[index].flags != fill)
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				SetSelection(i, true);
 			else
-				SetItemState(i, 0, wxLIST_STATE_SELECTED);
+				SetSelection(i, false);
 		}
+		if (m_pFilelistStatusBar)
+			m_pFilelistStatusBar->SelectAll();
 	}
 	else
 		OnChar(event);
@@ -1572,6 +1660,11 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 				else
 					permissions[i] = -1;
 			}
+			if (entry.permissions[6] == 's')
+				permissions[5] = 2;
+			if (entry.permissions[9] == 't')
+				permissions[8] = 2;
+
 		}
 	}
 	if (!dirCount && !fileCount)
@@ -1677,23 +1770,55 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 
 void CRemoteListView::ApplyCurrentFilter()
 {
+	CFilterManager filter;
+
+	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing())
+		ExitComparisonMode();
+
 	if (m_fileData.size() <= 1)
 		return;
 
 	wxString focused;
 	std::list<wxString> selectedNames = RememberSelectedItems(focused);
 
-	CFilterDialog filter;
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->UnselectAll();
+
+	wxLongLong totalSize;
+	int unknown_sizes = 0;
+	int totalFileCount = 0;
+	int totalDirCount = 0;
+	int hidden = 0;
+
 	m_indexMapping.clear();
 	const unsigned int count = m_pDirectoryListing->GetCount();
 	m_indexMapping.push_back(count);
 	for (unsigned int i = 0; i < count; i++)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[i];
-		if (!filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
-			m_indexMapping.push_back(i);
+		if (filter.FilenameFiltered(entry.name, entry.dir, entry.size, false, 0))
+		{
+			hidden++;
+			continue;
+		}
+
+		if (entry.dir)
+			totalDirCount++;
+		else
+		{
+			if (entry.size == -1)
+				unknown_sizes++;
+			else
+				totalSize += entry.size;
+			totalFileCount++;
+		}
+
+		m_indexMapping.push_back(i);
 	}
 
+	if (m_pFilelistStatusBar)
+		m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+
 	SetItemCount(m_indexMapping.size());
 
 	SortList(-1, -1, false);
@@ -1717,7 +1842,7 @@ std::list<wxString> CRemoteListView::RememberSelectedItems(wxString& focused)
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	while (item != -1)
 	{
-		SetItemState(item, 0, wxLIST_STATE_SELECTED);
+		SetSelection(item, false);
 		if (!item)
 		{
 			selectedNames.push_back(_T(".."));
@@ -1817,14 +1942,18 @@ void CRemoteListView::ReselectItems(std::list<wxString>& selectedNames, wxString
 			{
 				if (firstSelected == -1)
 					firstSelected = i;
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				if (m_pFilelistStatusBar)
+					m_pFilelistStatusBar->SelectDirectory();
+                SetSelection(i, true);
 				break;
 			}
 			else if (*iter == (_T("-") + entry.name))
 			{
 				if (firstSelected == -1)
 					firstSelected = i;
-				SetItemState(i, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
+				if (m_pFilelistStatusBar)
+					m_pFilelistStatusBar->SelectFile(entry.size);
+				SetSelection(i, true);
 				break;
 			}
 		}
@@ -1868,18 +1997,26 @@ void CRemoteListView::RepositionInfoText()
 	rect.height = size.y;
 
 	m_pInfoText->SetSize(rect);
+#ifdef __WXMSW__
+	m_pInfoText->Refresh(true);
+	m_pInfoText->Update();
+#else
 	m_pInfoText->Refresh(false);
+#endif
 }
 
-void CRemoteListView::OnStateChange(unsigned int event, const wxString& data)
+void CRemoteListView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
 {
 	wxASSERT(m_pState);
-	if (event == STATECHANGE_REMOTE_DIR)
+	if (notification == STATECHANGE_REMOTE_DIR)
 		SetDirectoryListing(m_pState->GetRemoteDir(), false);
-	else if (event == STATECHANGE_REMOTE_DIR_MODIFIED)
+	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
 		SetDirectoryListing(m_pState->GetRemoteDir(), true);
-	else if (event == STATECHANGE_APPLYFILTER)
+	else
+	{
+		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		ApplyCurrentFilter();
+	}
 }
 
 void CRemoteListView::SetInfoText()
@@ -1905,6 +2042,10 @@ void CRemoteListView::SetInfoText()
 	if (!m_pInfoText)
 	{
 		m_pInfoText = new CInfoText(this, text);
+#ifdef __WXMSW__
+		m_pInfoText->SetDoubleBuffered(true);
+#endif
+
 		RepositionInfoText();
 		return;
 	}
@@ -2215,6 +2356,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 	case CEditHandler::download:
 	case CEditHandler::upload:
 	case CEditHandler::upload_and_remove:
+	case CEditHandler::upload_and_remove_failed:
 		wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view / edit selected file"), wxICON_EXCLAMATION);
 		return;
 	case CEditHandler::removing:
@@ -2387,6 +2529,11 @@ wxListItemAttr* CRemoteListView::OnGetItemAttr(long item) const
 	if (index == -1)
 		return 0;
 
+#ifndef __WXMSW__
+	if (item == m_dropTarget)
+		return &pThis->m_dropHighlightAttribute;
+#endif
+
 	const CGenericFileData& data = m_fileData[index];
 
 	switch (data.flags)
@@ -2405,7 +2552,32 @@ wxListItemAttr* CRemoteListView::OnGetItemAttr(long item) const
 }
 
 // Defined in LocalListView.cpp
-extern wxString FormatSize(const wxLongLong& size);
+extern wxString FormatSize(const wxLongLong& size, bool add_bytes_suffix = false);
+
+// Filenames on VMS systems have a revision suffix, e.g.
+// foo.bar;1
+// foo.bar;2
+// foo.bar;3
+wxString StripVMSRevision(const wxString& name)
+{
+	int pos = name.Find(';', true);
+	if (pos < 1)
+		return name;
+
+	const int len = name.Len();
+	if (pos == len - 1)
+		return name;
+
+	int p = pos;
+	while (++p < len)
+	{
+		const wxChar& c = name[p];
+		if (c < '0' || c > '9')
+			return name;
+	}
+
+	return name.Left(pos);
+}
 
 wxString CRemoteListView::GetItemText(int item, unsigned int column)
 {
@@ -2442,7 +2614,10 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		if (data.fileType == _T(""))
 		{
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
-			data.fileType = GetType(entry.name, entry.dir);
+			if (m_pDirectoryListing->path.GetType() == VMS)
+				data.fileType = GetType(StripVMSRevision(entry.name), entry.dir);
+			else
+				data.fileType = GetType(entry.name, entry.dir);
 		}
 
 		return data.fileType;
@@ -2502,3 +2677,13 @@ void CRemoteListView::OnExitComparisonMode()
 	CFileListCtrl<CGenericFileData>::OnExitComparisonMode();
 	SetInfoText();
 }
+
+bool CRemoteListView::ItemIsDir(int index) const
+{
+	return (*m_pDirectoryListing)[index].dir;
+}
+
+wxLongLong CRemoteListView::ItemGetSize(int index) const
+{
+	return (*m_pDirectoryListing)[index].size;
+}
