@@ -8,6 +8,7 @@
 class CQueueView;
 class CFileListCtrl_SortComparisonObject;
 class CState;
+class CFilelistStatusBar;
 
 class CGenericFileData
 {
@@ -61,6 +62,8 @@ public:
 		CListViewSort* m_pObject;
 	};
 
+	void SetFilelistStatusBar(CFilelistStatusBar* pFilelistStatusBar) { m_pFilelistStatusBar = pFilelistStatusBar; }
+
 protected:
 	CQueueView *m_pQueue;
 
@@ -68,6 +71,9 @@ protected:
 	std::vector<unsigned int> m_indexMapping;
 	std::vector<unsigned int> m_originalIndexMapping; // m_originalIndexMapping will only be set on comparisons
 
+	virtual bool ItemIsDir(int index) const = 0;
+	virtual wxLongLong ItemGetSize(int index) const = 0;
+
 	std::map<wxString, wxString> m_fileTypeMap;
 
 #ifdef __WXMSW__
@@ -82,6 +88,7 @@ protected:
 	int m_sortColumn;
 	int m_sortDirection;
 
+	void InitSort(int optionID); // Has to be called after initializing columns
 	void SortList(int column = -1, int direction = -1, bool updateSelections = true);
 	enum CListViewSort::DirSortMode GetDirSortMode();
 	virtual CSortComparisonObject GetSortComparisonObject() = 0;
@@ -103,11 +110,47 @@ protected:
 	void ComparisonRestoreSelections();
 	std::list<int> m_comparisonSelections;
 
+	CFilelistStatusBar* m_pFilelistStatusBar;
+
+#ifndef __WXMSW__
+	// Generic wxListCtrl does not support wxLIST_STATE_DROPHILITED, emulate it
+	wxListItemAttr m_dropHighlightAttribute;
+#endif
+
+	void SetSelection(int item, bool select);
+#ifndef __WXMSW__
+	// Used by selection tracking
+	void SetItemCount(int count);
+#endif
+
 private:
 	void SortList_UpdateSelections(bool* selections, int focus);
 
+	// If this is set to true, don't process selection changed events
+	bool m_insideSetSelection;
+
+#ifdef __WXMSW__
+	static LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
+	WNDPROC m_prevWndproc;
+	// char* instead of CFileListCtrl<CFileData> due to bug in mingw compiler
+	static std::map<HWND, char*> m_hwnd_map;
+#else
+	int m_focusItem;
+	std::vector<bool> m_selections;
+	int m_pending_focus_processing;
+#endif
+
 	DECLARE_EVENT_TABLE()
 	void OnColumnClicked(wxListEvent &event);
+	void OnColumnRightClicked(wxListEvent& event);
+	void OnItemSelected(wxListEvent& event);
+	void OnItemDeselected(wxListEvent& event);
+#ifndef __WXMSW__
+	void OnFocusChanged(wxListEvent& event);
+	void OnProcessFocusChange(wxCommandEvent& event);
+	void OnLeftDown(wxMouseEvent& event);
+	void OnProcessMouseEvent(wxCommandEvent& event);
+#endif
 };
 
 #ifdef FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
