@@ -8,6 +8,9 @@
 #include "buildinfo.h"
 #include <wx/stdpaths.h>
 #include "Mainfrm.h"
+#ifdef __WXMSW__
+#include <wx/dynlib.h> // Used by GetDownloadDir
+#endif //__WXMSW__
 
 #define MAXCHECKPROGRESS 9 // Maximum value of progress bar
 
@@ -39,6 +42,11 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 	version.Replace(_T(" "), _T("%20"));
 	m_urlServer = _T("update.filezilla-project.org");
 	m_urlFile = wxString::Format(_T("/updatecheck.php?platform=%s&version=%s"), host.c_str(), version.c_str());
+#if defined(__WXMSW__) || defined(__WXMAC__)
+	// Makes not much sense to submit OS version on Linux, *BSD and the likes, too many flavours.
+	wxString osVersion = wxString::Format(_T("&osversion=%d.%d"), wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
+	m_urlFile += osVersion;
+#endif
 
 	m_statusTimer.SetOwner(this, 0);
 	m_autoCheckTimer.SetOwner(this, 1);
@@ -47,6 +55,7 @@ CUpdateWizard::CUpdateWizard(wxWindow* pParent)
 
 	m_loaded = false;
 	m_updateShown = false;
+	m_start_check = false;
 }
 
 CUpdateWizard::~CUpdateWizard()
@@ -101,6 +110,11 @@ bool CUpdateWizard::Load()
 		GetPageAreaSizer()->Add(m_pages[i]);
 	}
 
+	if (COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0)
+		XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->SetValue(1);
+	else
+		XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->SetValue(0);
+
 	m_loaded = true;
 
 	CenterOnParent();
@@ -122,25 +136,33 @@ bool CUpdateWizard::Run()
 	if (CBuildInfo::ConvertToVersionNumber(newVersion) <= CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion()))
 	{
 		pOptions->SetOption(OPTION_UPDATECHECK_NEWVERSION, _T(""));
-		pOptions->SetOption(OPTION_UPDATECHECK_URL, _T(""));
 		return RunWizard(m_pages.front());
 	}
 
-	PrepareUpdateAvailablePage(newVersion, pOptions->GetOption(OPTION_UPDATECHECK_URL));
+	// Force another check
+	PrepareUpdateCheckPage();
+	m_start_check = true;
+	m_currentPage = 0;
 
-	m_currentPage = 1;
-	return RunWizard(m_pages[1]);
+	return RunWizard(m_pages[0]);
 }
 
 void CUpdateWizard::OnCheck(wxCommandEvent& event)
 {
-	if (event.GetId() == XRCID("ID_CHECKBETA") && event.IsChecked())
+	if (event.GetId() == XRCID("ID_CHECKBETA"))
 	{
-		if (wxMessageBox(_("Do you really want to check for beta versions?\nUnless you want to test new features, keep using stable versions."), _("Update wizard"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+		if (event.IsChecked())
 		{
-			XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->SetValue(0);
-			return;
+			if (wxMessageBox(_("Do you really want to check for beta versions?\nUnless you want to test new features, keep using stable versions."), _("Update wizard"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+			{
+				XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->SetValue(0);
+				return;
+			}
+			COptions::Get()->SetOption(OPTION_UPDATECHECK_CHECKBETA, 1);
 		}
+		else
+			COptions::Get()->SetOption(OPTION_UPDATECHECK_CHECKBETA, 0);
+
 	}
 	else if (event.GetId() == XRCID("ID_CHECKNIGHTLY") && event.IsChecked())
 	{
@@ -152,6 +174,37 @@ void CUpdateWizard::OnCheck(wxCommandEvent& event)
 	}
 }
 
+#ifdef __WXMSW__
+// See comment a few lines below
+GUID VISTASHIT_FOLDERID_Downloads = { 0x374de290, 0x123f, 0x4565, 0x91, 0x64, 0x39, 0xc4, 0x92, 0x5e, 0x46, 0x7b };
+extern "C" typedef HRESULT (WINAPI *tSHGetKnownFolderPath)(const GUID& rfid, DWORD dwFlags, HANDLE hToken, PWSTR *ppszPath);
+#endif
+
+wxString CUpdateWizard::GetDownloadDir()
+{
+#ifdef __WXMSW__
+	// Old Vista has a profile directory for downloaded files,
+	// need to get it using SHGetKnownFolderPath which we need to
+	// load dynamically to preserve forward compatibility with the 
+	// upgrade to Windows XP.
+	wxDynamicLibrary lib(_T("shell32.dll"));
+	if (lib.IsLoaded() && lib.HasSymbol(_T("SHGetKnownFolderPath")))
+	{
+		tSHGetKnownFolderPath pSHGetKnownFolderPath = (tSHGetKnownFolderPath)lib.GetSymbol(_T("SHGetKnownFolderPath"));
+
+		PWSTR path;
+		HRESULT result = pSHGetKnownFolderPath(VISTASHIT_FOLDERID_Downloads, 0, 0, &path);
+		if (result == S_OK)
+		{
+			wxString dir = path;
+			CoTaskMemFree(path);
+			return dir;
+		}
+	}
+#endif
+	return wxStandardPaths::Get().GetDocumentsDir();
+}
+
 void CUpdateWizard::OnPageChanging(wxWizardEvent& event)
 {
 	if (m_skipPageChanging)
@@ -160,22 +213,9 @@ void CUpdateWizard::OnPageChanging(wxWizardEvent& event)
 
 	if (event.GetPage() == m_pages[0])
 	{
-		XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->Disable();
-		XRCCTRL(*this, "ID_CHECKNIGHTLY", wxCheckBox)->Disable();
-		wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
-		pNext->Disable();
-
-		XRCCTRL(*this, "ID_CHECKINGTEXT", wxStaticText)->Show();
-		wxGauge* pProgress = XRCCTRL(*this, "ID_CHECKINGPROGRESS", wxGauge);
-		pProgress->Show();
-
-		wxStaticText *pText = XRCCTRL(*this, "ID_CHECKINGTEXTPROGRESS", wxStaticText);
-		pText->Show();
-		pText->SetLabel(_("Resolving hostname"));
-
-
 		event.Veto();
 
+		PrepareUpdateCheckPage();
 		StartUpdateCheck();
 	}
 	if (event.GetPage() == m_pages[1] && m_pages[1]->GetNext())
@@ -185,7 +225,7 @@ void CUpdateWizard::OnPageChanging(wxWizardEvent& event)
 		if (pos != -1)
 			filename = filename.Mid(pos + 1);
 
-		const wxString defaultDir = wxStandardPaths::Get().GetDocumentsDir();
+		const wxString defaultDir = GetDownloadDir();
 
 		const int flags = wxFD_SAVE | wxFD_OVERWRITE_PROMPT;
 
@@ -215,6 +255,16 @@ void CUpdateWizard::OnPageChanging(wxWizardEvent& event)
 
 void CUpdateWizard::OnPageChanged(wxWizardEvent& event)
 {
+	if (event.GetPage() == m_pages[0])
+	{
+		if (m_start_check)
+		{
+			m_start_check = false;
+			StartUpdateCheck();
+		}
+		return;
+	}
+
 	if (event.GetPage() != m_pages[2])
 		return;
 
@@ -230,16 +280,8 @@ void CUpdateWizard::OnPageChanged(wxWizardEvent& event)
 	int res = m_pEngine->Command(CConnectCommand(CServer(HTTP, DEFAULT, m_urlServer, 80)));
 	if (res == FZ_REPLY_OK)
 	{
-		m_inTransfer = true;
 		XRCCTRL(*this, "ID_DOWNLOADPROGRESSTEXT", wxStaticText)->SetLabel(_("Connecting to server"));
-		CFileTransferCommand::t_transferSettings transferSettings;
-
-		CServerPath path;
-		wxString file = m_urlFile;
-		path.SetPath(file, true);
-
-		CFileTransferCommand cmd(m_localFile, path, file, true, transferSettings);
-		res = m_pEngine->Command(cmd);
+		res = SendTransferCommand();
 
 		XRCCTRL(*this, "ID_DOWNLOADPROGRESS", wxGauge)->SetRange(100);
 	}
@@ -344,22 +386,13 @@ void CUpdateWizard::OnEngineEvent(wxEvent& event)
 				}
 				if (!m_inTransfer)
 				{
-					m_inTransfer = true;
-
 					if (m_loaded && !m_currentPage)
 					{
 						wxGauge* pProgress = XRCCTRL(*this, "ID_CHECKINGPROGRESS", wxGauge);
 						pProgress->SetValue(pProgress->GetValue() + 1);
 					}
 
-					CFileTransferCommand::t_transferSettings transferSettings;
-
-					CServerPath path;
-					wxString file = m_urlFile;
-					path.SetPath(file, true);
-
-					CFileTransferCommand cmd(m_localFile, path, file, true, transferSettings);
-					int res = m_pEngine->Command(cmd);
+					int res = SendTransferCommand();
 					if (res == FZ_REPLY_WOULDBLOCK)
 						break;
 					else if (res != FZ_REPLY_OK)
@@ -477,6 +510,15 @@ void CUpdateWizard::ParseData()
 
 		line.Trim(true);
 		line.Trim(false);
+
+		if (line == _T(""))
+		{
+			// After empty line, changelog follows
+			m_news = m_data;
+			m_news.Trim(true);
+			m_news.Trim(false);
+			break;
+		}
 		
 		// Extract type of update
 		pos = line.Find(' ');
@@ -530,30 +572,6 @@ void CUpdateWizard::ParseData()
 			if (CBuildInfo::ConvertToVersionNumber(versionOrDate) <= ownVersionNumber)
 				continue;
 		}
-		if (type == _T("beta"))
-		{
-			// Update beta only if at least one of these conditions is true:
-			// - Beta box is checked
-			// - Current version is a beta and no newer stable version exists
-
-			const bool checkBeta = m_loaded ? XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->GetValue() : false;
-
-			if (!checkBeta &&
-				((ownVersionNumber & 0x07FFFF) == 0 || newVersionNumber != 0))
-				continue;
-
-			if (newVersionNumber >= CBuildInfo::ConvertToVersionNumber(versionOrDate))
-				continue;
-		}
-		else
-		{
-			// Final releases
-			if (m_loaded)
-			{
-				if (CBuildInfo::ConvertToVersionNumber(versionOrDate) < newVersionNumber && XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->GetValue())
-					continue;
-			}
-		}
 
 		newVersion = versionOrDate;
 		newVersionNumber = CBuildInfo::ConvertToVersionNumber(versionOrDate);
@@ -567,12 +585,18 @@ void CUpdateWizard::ParseData()
 
 		COptions* pOptions = COptions::Get();
 		pOptions->SetOption(OPTION_UPDATECHECK_NEWVERSION, newVersion);
-		pOptions->SetOption(OPTION_UPDATECHECK_URL, newUrl);
 
 		DisplayUpdateAvailability(true);
 
 		return;
 	}
+	else
+	{
+		// Since the auto check and the manual check, a newer version might have been published
+		COptions* pOptions = COptions::Get();
+		if (!pOptions->GetOption(OPTION_UPDATECHECK_NEWVERSION).empty())
+			pOptions->SetOption(OPTION_UPDATECHECK_NEWVERSION, newVersion);
+	}
 
 	if (newVersion == _T(""))
 	{
@@ -747,24 +771,20 @@ bool CUpdateWizard::CanAutoCheckForUpdateNow()
 void CUpdateWizard::StartUpdateCheck()
 {
 	m_inTransfer = false;
+
+	if (COptions::Get()->GetOptionVal(OPTION_UPDATECHECK_CHECKBETA) != 0)
+		m_urlFile += _T("&beta=1");
+
 	int res = m_pEngine->Command(CConnectCommand(CServer(HTTP, DEFAULT, m_urlServer, 80)));
 	if (res == FZ_REPLY_OK)
 	{
-		m_inTransfer = true;
 		if (m_loaded)
 		{
 			XRCCTRL(*this, "ID_CHECKINGTEXTPROGRESS", wxStaticText)->SetLabel(_("Connecting to server"));
 			wxGauge* pProgress = XRCCTRL(*this, "ID_CHECKINGPROGRESS", wxGauge);
 			pProgress->SetValue(pProgress->GetValue() + 1);
 		}
-		CFileTransferCommand::t_transferSettings transferSettings;
-
-		CServerPath path;
-		wxString file = m_urlFile;
-		path.SetPath(file, true);
-
-		CFileTransferCommand cmd(_T(""), path, file, true, transferSettings);
-		res = m_pEngine->Command(cmd);
+		res = SendTransferCommand();
 	}
 	wxASSERT(res != FZ_REPLY_OK);
 	if (res != FZ_REPLY_WOULDBLOCK)
@@ -788,10 +808,14 @@ void CUpdateWizard::DisplayUpdateAvailability(bool showDialog, bool forceMenu /*
 	if (CBuildInfo::ConvertToVersionNumber(newVersion) <= CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion()))
 	{
 		pOptions->SetOption(OPTION_UPDATECHECK_NEWVERSION, _T(""));
-		pOptions->SetOption(OPTION_UPDATECHECK_URL, _T(""));
 		return;
 	}
 
+#ifdef __WXMSW__
+	// All open menus need to be closed or app will become unresponsive.
+	::EndMenu();
+#endif
+
 	m_updateShown = true;
 
 	CMainFrame* pFrame = (CMainFrame*)m_parent;
@@ -799,7 +823,9 @@ void CUpdateWizard::DisplayUpdateAvailability(bool showDialog, bool forceMenu /*
 	wxMenu* pMenu = new wxMenu();
 	const wxString& name = wxString::Format(_("&Version %s"), pOptions->GetOption(OPTION_UPDATECHECK_NEWVERSION).c_str());
 	pMenu->Append(XRCID("ID_CHECKFORUPDATES"), name);
-	pFrame->GetMenuBar()->Append(pMenu, _("&New version available!"));
+	wxMenuBar* pMenuBar = pFrame->GetMenuBar();
+	if (pMenuBar)
+		pMenuBar->Append(pMenu, _("&New version available!"));
 
 	if (showDialog)
 	{
@@ -820,6 +846,9 @@ void CUpdateWizard::PrepareUpdateAvailablePage(const wxString &newVersion, wxStr
 		else
 			XRCCTRL(*this, "ID_UPDATEDESC", wxStaticText)->SetLabel(_("Please check the package manager of your system for an updated package or visit http://filezilla-project.org to download the source code of FileZilla."));
 		XRCCTRL(*this, "ID_UPDATEDESC2", wxStaticText)->SetLabel(_T(""));
+
+		XRCCTRL(*this, "ID_NEWS_DESC", wxStaticText)->Hide();
+		XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->Hide();
 		m_pages[1]->SetNext(0);
 	}
 	else
@@ -842,6 +871,13 @@ void CUpdateWizard::PrepareUpdateAvailablePage(const wxString &newVersion, wxStr
 			m_urlServer = newUrl.Left(pos);
 			m_urlFile = newUrl.Mid(pos);
 		}
+		if (m_news == _T(""))
+		{
+			XRCCTRL(*this, "ID_NEWS_DESC", wxStaticText)->Hide();
+			XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->Hide();
+		}
+		else
+			XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->ChangeValue(m_news);
 	}
 	RewrapPage(1);
 }
@@ -862,4 +898,34 @@ void CUpdateWizard::RewrapPage(int page)
 	m_pages[page]->GetSizer()->Fit(m_pages[page]);
 }
 
+int CUpdateWizard::SendTransferCommand()
+{
+	m_inTransfer = true;
+
+	CFileTransferCommand::t_transferSettings transferSettings;
+
+	CServerPath path;
+	wxString file = m_urlFile;
+	path.SetPath(file, true);
+
+	CFileTransferCommand cmd(m_localFile, path, file, true, transferSettings);
+	return m_pEngine->Command(cmd);
+}
+
+void CUpdateWizard::PrepareUpdateCheckPage()
+{
+	XRCCTRL(*this, "ID_CHECKBETA", wxCheckBox)->Disable();
+	XRCCTRL(*this, "ID_CHECKNIGHTLY", wxCheckBox)->Disable();
+	wxButton* pNext = wxDynamicCast(FindWindow(wxID_FORWARD), wxButton);
+	pNext->Disable();
+
+	XRCCTRL(*this, "ID_CHECKINGTEXT", wxStaticText)->Show();
+	wxGauge* pProgress = XRCCTRL(*this, "ID_CHECKINGPROGRESS", wxGauge);
+	pProgress->Show();
+
+	wxStaticText *pText = XRCCTRL(*this, "ID_CHECKINGTEXTPROGRESS", wxStaticText);
+	pText->Show();
+	pText->SetLabel(_("Resolving hostname"));
+}
+
 #endif //FZ_MANUALUPDATECHECK
